@@ -1,18 +1,18 @@
-from . import simple_ga
+from .simple_Model import Model as SimpleModel
+# from .intermediate_Model import Model as IntermediateModel
+# from .advanced_Model import Model as AdvancedModel
 
-# import .intermediate_ga
-# import .advanced_ga
 from . import GA_Support
 
 
 def backend(config):
     if config["backend_type"] == "high":
-        return simple_ga.Model(config)
+        return SimpleModel(config)
     elif config["backend_type"] == "mid":
-        # return .intermediate_ga.Model(config)
+        # return IntermediateModel(config)
         pass
     elif config["backend_type"] == "low":
-        # return .advanced_ga.Model(config)
+        # return AdvancedModel(config)
         pass
     else:
         raise GA_Support.UnsupportedBackendDecorator(config["backend_type"])
