@@ -12,7 +12,7 @@ class Individual(GA_Individual):
     Data structure for the ansatz.
     """
 
-    def __init__(self, n_qubits, n_moments, genepool, rng_seed):
+    def __init__(self, n_qubits, n_moments, genepool, rng_seed, ansatz=None):
         """
         Initializes the inidivual object, all individuals (ansatz) are members of this class.
             - n_qubits: Number of qubits in the ansatz.
@@ -33,7 +33,10 @@ class Individual(GA_Individual):
         self.ansatz_draw = []
         self.params = []
 
-        self.generate()  # Should this be done automatically?
+        if ansatz is None:
+            self.generate()
+        else:
+            self.generate_from(ansatz)
         self.convert_to_qml()
         self.draw_ansatz()
 
@@ -95,6 +98,11 @@ class Individual(GA_Individual):
                 else:
                     raise Exception("Gates with more than 2 qubits haven't been implemented yet.")
 
+    def generate_from(self, ansatz):
+        self.ansatz_dicts = copy.deepcopy(ansatz)
+        self.convert_to_qml()
+        self.ansatz_draw()
+
     def convert_to_qml(self):
         """
         Converts the ansatz into a general format for the QML.
@@ -228,18 +236,17 @@ class Individual(GA_Individual):
                 if self.ansatz_dicts[moment][qubit_pair].find("_") > 0:
                     double_swap_flag += 1
                     gate_pair = self.genepool.choice()
-                    if double_swap_flag == 2 and gate_pair.n_qubits == 1:
+                    if double_swap_flag == 2 and gate_pair.n_qubits == 2:
                         direction = self.rng.permutation(["_C", "_T"])
                         self.ansatz_dicts[moment][int(self.ansatz_dicts[moment][qubit][-1])] = (
-                            gate.name + direction[0] + f"-{int(self.ansatz_dicts[moment][qubit_pair][-1])}"
+                            gate_pair.name + direction[0] + f"-{int(self.ansatz_dicts[moment][qubit_pair][-1])}"
                         )
                         self.ansatz_dicts[moment][int(self.ansatz_dicts[moment][qubit_pair][-1])] = (
-                            gate.name + direction[1] + f"-{int(self.ansatz_dicts[moment][qubit][-1])}"
+                            gate_pair.name + direction[1] + f"-{int(self.ansatz_dicts[moment][qubit][-1])}"
                         )
                     else:
-                        self.ansatz_dicts[moment][int(self.ansatz_dicts[moment][qubit_pair][-1])] = self.rng.choice(
-                            self.gates_arr[:-1]
-                        )
+                        gate_pair = self.genepool.choice(n_qubits=1)
+                        self.ansatz_dicts[moment][int(self.ansatz_dicts[moment][qubit_pair][-1])] = gate_pair.name
 
                 direction = self.rng.permutation(["_C", "_T"])
                 self.ansatz_dicts[moment][qubit] = gate.name + direction[0] + f"-{qubit_pair}"
