@@ -45,6 +45,7 @@ class Model(GA_Model):
         self.best_perf = {
             "fitness": 0,
             "eval_metrics": [],
+            "ansatz": None,
             "ansatz_dicts": [],
             "ansatz_draw": str(),
             "generation": 0,
@@ -137,15 +138,39 @@ class Model(GA_Model):
             if gen > 20:
                 if (gen - self.best_perf["generation"]) > self.n_steps_patience:
                     break
-            make_results_json(results, self.start_time, self.ga_output_path, gen)
+            print(
+                "filepath is: ",
+                make_results_json(results, self.start_time, self.ga_output_path, gen)
+            )
             gen += 1
         print(
-            "filename is: ",
+            "filepath is: ",
             make_results_json(
                 results, self.start_time, self.ga_output_path, gen, final_flag=True
             ),
         )
 
+        ### Final re-training for std. dev. estimate ###
+        ansatz = self.best_perf["ansatz"]
+        ansatz.convert_to_qml()
+        ansatz.draw_ansatz()
+        vqc_config_ansatz = {key: value for key, value in self.vqc_config.items()}
+        vqc_config_ansatz["ansatz_dicts"] = ansatz.ansatz_dicts
+        vqc_config_ansatz["ansatz_qml"] = ansatz.ansatz_qml
+        vqc_config_ansatz["params"] = ansatz.params
+        vqc_config_ansatz["gen"] = gen
+        fitness_arr = []
+        auroc_arr = []
+        for i in range(20):
+            vqc_config_ansatz["ix"] = i
+            output_dict = self.vqc(vqc_config_ansatz)
+            fitness_arr.append(output_dict["fitness_metric"])
+            auroc_arr.append(output_dict["eval_metrics"]["auroc"])
+        print(f"Final fitness distribution: {fitness_arr}")
+        print(f"Avg fitness: {np.mean(fitness_arr)},  Std Dev: {np.std(fitness_arr)}, Std Dev of Mean: {np.std(fitness_arr) / (20**0.5)}")
+        print(f"Final AUROC distribution: {auroc_arr}")
+        print(f"Avg AUROC: {np.mean(auroc_arr)},  Std Dev: {np.std(auroc_arr)}, Std Dev of Mean: {np.std(auroc_arr) / (20**0.5)}")
+
     def evaluate_fitness(self, gen):
         """
         Evaluates the fitness level of all ansatz. Runs the QML optimization task.
@@ -194,6 +219,9 @@ class Model(GA_Model):
             self.best_perf["eval_metrics"] = copy.deepcopy(
                 self.metrics_arr[np.argmax(self.fitness_arr)]
             )
+            self.best_perf["ansatz"] = copy.deepcopy(
+                self.population[np.argmax(self.fitness_arr)]
+            )
             self.best_perf["ansatz_dicts"] = copy.deepcopy(
                 self.population[np.argmax(self.fitness_arr)].ansatz_dicts
             )
@@ -206,13 +234,13 @@ class Model(GA_Model):
     def make_results(self, gen):
         ### Euclidean distances ###
         distances_from_best = euclidean_distances(self.population[np.argmax(self.fitness_arr)], self.population)
-        destdir_curves = os.path.join(self.ga_output_path, "ga_curves")
+        destdir_curves = os.path.join(self.ga_output_path, "ga_curves", "run-%s" % self.start_time)
         if not os.path.exists(destdir_curves):
             os.makedirs(destdir_curves)
         filepath_euclid = os.path.join(
             destdir_curves,
-            "%03deuclid_distance-%s_data.png"
-            % (gen, self.start_time),
+            "%03deuclid_distance_data.png"
+            % gen
         )
         plt.figure(0)
         plt.style.use("seaborn")
@@ -227,10 +255,10 @@ class Model(GA_Model):
         x, y = data_tsne[0], data_tsne[1]
         filepath_tsne = os.path.join(
             destdir_curves,
-            "%03dtsne_distance-%s_data.png"
-            % (gen, self.start_time),
+            "%03dtsne_distance_data.png"
+            % gen
         )
-        plt.figure(0)
+        plt.figure(1)
         plt.style.use("seaborn")
         plt.scatter(x, y, marker=".", c=[i["auroc"] for i in self.metrics_arr], cmap=plt.set_cmap('plasma'))
         plt.ylabel("a.u.")
@@ -269,7 +297,7 @@ class Model(GA_Model):
         Picks the top performing ansatz from a generation to mate and mutate for the next generation.
         """
         winner_arr = []
-        for i in range(self.n_winners):
+        for _ in range(self.n_winners):
             winner_ix = np.argmax(self.fitness_arr)
             winner = self.population[winner_ix]
             winner_arr.append(winner)
